"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSyncGeneratorChanges = getSyncGeneratorChanges;
exports.clearSyncGeneratorChanges = clearSyncGeneratorChanges;
exports.collectAllRegisteredSyncGenerators = collectAllRegisteredSyncGenerators;
exports.runSyncGenerator = runSyncGenerator;
exports.collectRegisteredTaskSyncGenerators = collectRegisteredTaskSyncGenerators;
exports.collectRegisteredGlobalSyncGenerators = collectRegisteredGlobalSyncGenerators;
exports.syncGeneratorResultsToMessageLines = syncGeneratorResultsToMessageLines;
const generate_1 = require("../command-line/generate/generate");
const generator_utils_1 = require("../command-line/generate/generator-utils");
const nx_json_1 = require("../config/nx-json");
const client_1 = require("../daemon/client/client");
const tree_1 = require("../generators/tree");
const project_graph_1 = require("../project-graph/project-graph");
const workspace_root_1 = require("./workspace-root");
const chalk = require("chalk");
async function getSyncGeneratorChanges(generators) {
    let results;
    if (!client_1.daemonClient.enabled()) {
        results = await runSyncGenerators(generators);
    }
    else {
        results = await client_1.daemonClient.getSyncGeneratorChanges(generators);
    }
    return results.filter((r) => r.changes.length > 0);
}
async function clearSyncGeneratorChanges(generators) {
    if (client_1.daemonClient.enabled()) {
        await client_1.daemonClient.clearSyncGeneratorChanges(generators);
    }
}
async function collectAllRegisteredSyncGenerators(projectGraph) {
    if (!client_1.daemonClient.enabled()) {
        return [
            ...collectRegisteredTaskSyncGenerators(projectGraph),
            ...collectRegisteredGlobalSyncGenerators(),
        ];
    }
    return await client_1.daemonClient.getRegisteredSyncGenerators();
}
async function runSyncGenerator(tree, generatorSpecifier, projects) {
    const { collection, generator } = (0, generate_1.parseGeneratorString)(generatorSpecifier);
    const { implementationFactory } = (0, generator_utils_1.getGeneratorInformation)(collection, generator, workspace_root_1.workspaceRoot, projects);
    const implementation = implementationFactory();
    const result = await implementation(tree, {});
    let callback;
    let outOfSyncMessage;
    if (result) {
        if (typeof result === 'function') {
            callback = result;
        }
        else {
            callback = result.callback;
            outOfSyncMessage = result.outOfSyncMessage;
        }
    }
    if (callback) {
        await callback();
    }
    return {
        changes: tree.listChanges(),
        generatorName: generatorSpecifier,
        outOfSyncMessage,
    };
}
function collectRegisteredTaskSyncGenerators(projectGraph) {
    const taskSyncGenerators = new Set();
    for (const { data: { targets }, } of Object.values(projectGraph.nodes)) {
        if (!targets) {
            continue;
        }
        for (const target of Object.values(targets)) {
            if (!target.syncGenerators) {
                continue;
            }
            for (const generator of target.syncGenerators) {
                taskSyncGenerators.add(generator);
            }
        }
    }
    return taskSyncGenerators;
}
function collectRegisteredGlobalSyncGenerators(nxJson = (0, nx_json_1.readNxJson)()) {
    const globalSyncGenerators = new Set();
    if (!nxJson.sync?.globalGenerators?.length) {
        return globalSyncGenerators;
    }
    for (const generator of nxJson.sync.globalGenerators) {
        globalSyncGenerators.add(generator);
    }
    return globalSyncGenerators;
}
function syncGeneratorResultsToMessageLines(results) {
    const messageLines = [];
    for (const result of results) {
        messageLines.push(`${chalk.bold(result.generatorName)}: ${chalk.bold(result.changes.length)} file(s) out of sync`);
        if (result.outOfSyncMessage) {
            messageLines.push(result.outOfSyncMessage);
        }
    }
    return messageLines;
}
async function runSyncGenerators(generators) {
    const tree = new tree_1.FsTree(workspace_root_1.workspaceRoot, false, 'running sync generators');
    const projectGraph = await (0, project_graph_1.createProjectGraphAsync)();
    const { projects } = (0, project_graph_1.readProjectsConfigurationFromProjectGraph)(projectGraph);
    const results = [];
    for (const generator of generators) {
        const result = await runSyncGenerator(tree, generator, projects);
        results.push(result);
    }
    return results;
}
